@@ -7,6 +7,7 @@ static void bspline_internal (double *y, const double *x, int nx, int i, int p,
 // B-spline basis
 
 SEXP bspline_basis (SEXP x, SEXP degree, SEXP knots) {
+  int nprotect = 0;
   SEXP y;
   int nx = length(x);
   int nknots = length(knots);
@@ -15,17 +16,18 @@ SEXP bspline_basis (SEXP x, SEXP degree, SEXP knots) {
   double *ydata;
   int i;
   if (deg < 0) error("must have degree > 0 in 'bspline.basis'");
-  PROTECT(y = allocMatrix(REALSXP,nx,nbasis));
+  PROTECT(y = allocMatrix(REALSXP,nx,nbasis)); nprotect++;
   ydata = REAL(y);
   for (i = 0; i < nbasis; i++) {
     bspline_internal(ydata,REAL(x),nx,i,deg,REAL(knots),nknots);
     ydata += nx;
   }
-  UNPROTECT(1);
+  UNPROTECT(nprotect);
   return(y);
 }
 
 SEXP bspline_basis_function (SEXP x, SEXP i, SEXP degree, SEXP knots) {
+  int nprotect = 0;
   SEXP y;
   int nx = length(x);
   int nknots = length(knots);
@@ -35,9 +37,9 @@ SEXP bspline_basis_function (SEXP x, SEXP i, SEXP degree, SEXP knots) {
   if ((ival < 0) || (deg < 0) || (ival + deg >= nknots-1)) 
     error("bad arguments in 'bspline'");
 
-  PROTECT(y = NEW_NUMERIC(nx));
+  PROTECT(y = NEW_NUMERIC(nx)); nprotect++;
   bspline_internal(REAL(y),REAL(x),nx,ival,deg,REAL(knots),nknots);
-  UNPROTECT(1);
+  UNPROTECT(nprotect);
   return(y);
 }
 
